@@ -547,9 +547,25 @@ static void
 on_relay_setup (GdmProductSlave *slave,
                 DBusMessage     *message)
 {
-        g_debug ("GdmProductSlave: Relay Setup");
+        DBusError   error;
+        const char *service_name;
+        dbus_bool_t res;
 
-        gdm_session_setup (GDM_SESSION (slave->priv->session));
+        service_name = NULL;
+
+        dbus_error_init (&error);
+        res = dbus_message_get_args (message,
+                                     &error,
+                                     DBUS_TYPE_STRING, &service_name,
+                                     DBUS_TYPE_INVALID);
+        if (res) {
+                g_debug ("GdmProductSlave: Relay Setup");
+                gdm_session_setup (GDM_SESSION (slave->priv->session),
+                                   service_name);
+        } else {
+                g_warning ("Unable to get arguments: %s", error.message);
+                dbus_error_free (&error);
+        }
 }
 
 static void
@@ -557,17 +573,24 @@ on_relay_setup_for_user (GdmProductSlave *slave,
                          DBusMessage     *message)
 {
         DBusError   error;
-        const char *text;
+        const char *service_name;
+        const char *username;
         dbus_bool_t res;
 
+        username = NULL;
+        service_name = NULL;
+
         dbus_error_init (&error);
         res = dbus_message_get_args (message,
                                      &error,
-                                     DBUS_TYPE_STRING, &text,
+                                     DBUS_TYPE_STRING, &service_name,
+                                     DBUS_TYPE_STRING, &username,
                                      DBUS_TYPE_INVALID);
         if (res) {
                 g_debug ("GdmProductSlave: Relay SetupForUser");
-                gdm_session_setup_for_user (GDM_SESSION (slave->priv->session), text);
+                gdm_session_setup_for_user (GDM_SESSION (slave->priv->session),
+                                            service_name,
+                                            username);
         } else {
                 g_warning ("Unable to get arguments: %s", error.message);
                 dbus_error_free (&error);
@@ -966,13 +989,12 @@ static gboolean
 gdm_product_slave_start (GdmSlave *slave)
 {
         gboolean ret;
-        gboolean res;
         GError  *error;
         char    *display_id;
 
         ret = FALSE;
 
-        res = GDM_SLAVE_CLASS (gdm_product_slave_parent_class)->start (slave);
+        GDM_SLAVE_CLASS (gdm_product_slave_parent_class)->start (slave);
 
         g_object_get (slave,
                       "display-id", &display_id,
@@ -1019,11 +1041,9 @@ gdm_product_slave_start (GdmSlave *slave)
 static gboolean
 gdm_product_slave_stop (GdmSlave *slave)
 {
-        gboolean res;
-
         g_debug ("GdmProductSlave: Stopping product_slave");
 
-        res = GDM_SLAVE_CLASS (gdm_product_slave_parent_class)->stop (slave);
+        GDM_SLAVE_CLASS (gdm_product_slave_parent_class)->stop (slave);
 
         if (GDM_PRODUCT_SLAVE (slave)->priv->session != NULL) {
                 gdm_session_close (GDM_SESSION (GDM_PRODUCT_SLAVE (slave)->priv->session));
@@ -1050,10 +1070,6 @@ gdm_product_slave_set_property (GObject      *object,
                                 const GValue *value,
                                 GParamSpec   *pspec)
 {
-        GdmProductSlave *self;
-
-        self = GDM_PRODUCT_SLAVE (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1067,10 +1083,6 @@ gdm_product_slave_get_property (GObject    *object,
                                 GValue     *value,
                                 GParamSpec *pspec)
 {
-        GdmProductSlave *self;
-
-        self = GDM_PRODUCT_SLAVE (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1084,9 +1096,6 @@ gdm_product_slave_constructor (GType                  type,
                                GObjectConstructParam *construct_properties)
 {
         GdmProductSlave      *product_slave;
-        GdmProductSlaveClass *klass;
-
-        klass = GDM_PRODUCT_SLAVE_CLASS (g_type_class_peek (GDM_TYPE_PRODUCT_SLAVE));
 
         product_slave = GDM_PRODUCT_SLAVE (G_OBJECT_CLASS (gdm_product_slave_parent_class)->constructor (type,
                                                                                                          n_construct_properties,
