@@ -42,6 +42,9 @@
 
 #include "gdm-common.h"
 
+#include "gdm-settings-client.h"
+#include "gdm-settings-keys.h"
+
 #include "gdm-simple-slave.h"
 #include "gdm-simple-slave-glue.h"
 
@@ -256,7 +259,7 @@ start_session_timeout (GdmSimpleSlave *slave)
                 goto out;
         }
 
-        gdm_greeter_session_stop (slave->priv->greeter);
+        gdm_welcome_session_stop (GDM_WELCOME_SESSION (slave->priv->greeter));
         gdm_greeter_server_stop (slave->priv->greeter_server);
 
         auth_file = NULL;
@@ -373,6 +376,25 @@ on_session_selected_user_changed (GdmSession     *session,
         gdm_greeter_server_selected_user_changed (slave->priv->greeter_server, text);
 }
 
+static void
+on_default_language_name_changed (GdmSession     *session,
+                                  const char     *text,
+                                  GdmSimpleSlave *slave)
+{
+        g_debug ("GdmSimpleSlave: Default language name changed: %s", text);
+
+        gdm_greeter_server_default_language_name_changed (slave->priv->greeter_server, text);
+}
+
+static void
+on_default_session_name_changed (GdmSession     *session,
+                                 const char     *text,
+                                 GdmSimpleSlave *slave)
+{
+        g_debug ("GdmSimpleSlave: Default session name changed: %s", text);
+
+        gdm_greeter_server_default_session_name_changed (slave->priv->greeter_server, text);
+}
 
 static void
 create_new_session (GdmSimpleSlave *slave)
@@ -381,6 +403,7 @@ create_new_session (GdmSimpleSlave *slave)
         char          *display_name;
         char          *display_hostname;
         char          *display_device;
+        char          *display_x11_authority_file;
 
         g_debug ("GdmSimpleSlave: Creating new session");
 
@@ -388,6 +411,7 @@ create_new_session (GdmSimpleSlave *slave)
                       "display-name", &display_name,
                       "display-hostname", &display_hostname,
                       "display-is-local", &display_is_local,
+                      "display-x11-authority-file", &display_x11_authority_file,
                       NULL);
 
         display_device = NULL;
@@ -398,6 +422,7 @@ create_new_session (GdmSimpleSlave *slave)
         slave->priv->session = gdm_session_direct_new (display_name,
                                                        display_hostname,
                                                        display_device,
+                                                       display_x11_authority_file,
                                                        display_is_local);
         g_free (display_name);
         g_free (display_device);
@@ -485,20 +510,49 @@ create_new_session (GdmSimpleSlave *slave)
                           "selected-user-changed",
                           G_CALLBACK (on_session_selected_user_changed),
                           slave);
+
+        g_signal_connect (slave->priv->session,
+                          "default-language-name-changed",
+                          G_CALLBACK (on_default_language_name_changed),
+                          slave);
+
+        g_signal_connect (slave->priv->session,
+                          "default-session-name-changed",
+                          G_CALLBACK (on_default_session_name_changed),
+                          slave);
 }
 
 static void
-on_greeter_start (GdmGreeterSession *greeter,
-                  GdmSimpleSlave    *slave)
+on_greeter_session_start (GdmGreeterSession *greeter,
+                          GdmSimpleSlave    *slave)
 {
         g_debug ("GdmSimpleSlave: Greeter started");
 }
 
 static void
-on_greeter_stop (GdmGreeterSession *greeter,
-                 GdmSimpleSlave    *slave)
+on_greeter_session_stop (GdmGreeterSession *greeter,
+                         GdmSimpleSlave    *slave)
 {
         g_debug ("GdmSimpleSlave: Greeter stopped");
+        gdm_slave_stopped (GDM_SLAVE (slave));
+}
+
+static void
+on_greeter_session_exited (GdmGreeterSession    *greeter,
+                           int                   code,
+                           GdmSimpleSlave       *slave)
+{
+        g_debug ("GdmSimpleSlave: Greeter exited: %d", code);
+        gdm_slave_stopped (GDM_SLAVE (slave));
+}
+
+static void
+on_greeter_session_died (GdmGreeterSession    *greeter,
+                         int                   signal,
+                         GdmSimpleSlave       *slave)
+{
+        g_debug ("GdmSimpleSlave: Greeter died: %d", signal);
+        gdm_slave_stopped (GDM_SLAVE (slave));
 }
 
 static void
@@ -506,7 +560,37 @@ on_greeter_begin_verification (GdmGreeterServer *greeter_server,
                                GdmSimpleSlave   *slave)
 {
         g_debug ("GdmSimpleSlave: begin verification");
-        gdm_session_setup (GDM_SESSION (slave->priv->session));
+        gdm_session_setup (GDM_SESSION (slave->priv->session),
+                           "gdm");
+}
+
+static void
+on_greeter_begin_timed_login (GdmGreeterServer *greeter_server,
+                              GdmSimpleSlave   *slave)
+{
+        char    *username;
+        gboolean enabled;
+        gboolean res;
+
+        g_debug ("GdmSimpleSlave: begin timed login");
+
+        enabled = FALSE;
+        res = gdm_settings_client_get_boolean (GDM_KEY_TIMED_LOGIN_ENABLE, &enabled);
+        if (! enabled) {
+                g_warning ("GdmSimpleSlave: timed login requested but disabled in configuration");
+                return;
+        }
+
+        username = NULL;
+        res = gdm_settings_client_get_string (GDM_KEY_TIMED_LOGIN_USER, &username);
+        if (username == NULL) {
+                g_warning ("GdmSimpleSlave: timed login requested but username not specified in configuration");
+                return;
+        }
+
+        gdm_session_setup_for_user (GDM_SESSION (slave->priv->session),
+                                    "gdm-autologin",
+                                    username);
 }
 
 static void
@@ -516,6 +600,7 @@ on_greeter_begin_verification_for_user (GdmGreeterServer *greeter_server,
 {
         g_debug ("GdmSimpleSlave: begin verification");
         gdm_session_setup_for_user (GDM_SESSION (slave->priv->session),
+                                    "gdm",
                                     username);
 }
 
@@ -636,11 +721,15 @@ run_greeter (GdmSimpleSlave *slave)
         }
 
         /* Run the init script. gdmslave suspends until script has terminated */
-        gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/Init", "gdm");
+        gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/Init/Default", "gdm");
 
         create_new_session (slave);
 
         slave->priv->greeter_server = gdm_greeter_server_new (display_id);
+        g_signal_connect (slave->priv->greeter_server,
+                          "begin-timed-login",
+                          G_CALLBACK (on_greeter_begin_timed_login),
+                          slave);
         g_signal_connect (slave->priv->greeter_server,
                           "begin-verification",
                           G_CALLBACK (on_greeter_begin_verification),
@@ -684,17 +773,25 @@ run_greeter (GdmSimpleSlave *slave)
                                                         display_is_local);
         g_signal_connect (slave->priv->greeter,
                           "started",
-                          G_CALLBACK (on_greeter_start),
+                          G_CALLBACK (on_greeter_session_start),
                           slave);
         g_signal_connect (slave->priv->greeter,
                           "stopped",
-                          G_CALLBACK (on_greeter_stop),
+                          G_CALLBACK (on_greeter_session_stop),
+                          slave);
+        g_signal_connect (slave->priv->greeter,
+                          "exited",
+                          G_CALLBACK (on_greeter_session_exited),
+                          slave);
+        g_signal_connect (slave->priv->greeter,
+                          "died",
+                          G_CALLBACK (on_greeter_session_died),
                           slave);
         g_object_set (slave->priv->greeter,
                       "x11-authority-file", auth_file,
                       NULL);
-        gdm_greeter_session_set_server_address (slave->priv->greeter, address);
-        gdm_greeter_session_start (slave->priv->greeter);
+        gdm_welcome_session_set_server_address (GDM_WELCOME_SESSION (slave->priv->greeter), address);
+        gdm_welcome_session_start (GDM_WELCOME_SESSION (slave->priv->greeter));
 
         g_free (display_id);
         g_free (display_name);
@@ -721,18 +818,41 @@ idle_connect_to_display (GdmSimpleSlave *slave)
                         g_warning ("Unable to connect to display after %d tries - bailing out", slave->priv->connection_attempts);
                         exit (1);
                 }
+                return TRUE;
         }
 
         return FALSE;
 }
 
 static void
-server_ready_cb (GdmServer      *server,
+on_server_ready (GdmServer      *server,
                  GdmSimpleSlave *slave)
 {
         g_timeout_add (500, (GSourceFunc)idle_connect_to_display, slave);
 }
 
+static void
+on_server_exited (GdmServer      *server,
+                  int             exit_code,
+                  GdmSimpleSlave *slave)
+{
+        g_debug ("GdmSimpleSlave: server exited with code %d\n", exit_code);
+
+        gdm_slave_stopped (GDM_SLAVE (slave));
+}
+
+static void
+on_server_died (GdmServer      *server,
+                int             signal_number,
+                GdmSimpleSlave *slave)
+{
+        g_debug ("GdmSimpleSlave: server died with signal %d, (%s)",
+                 signal_number,
+                 g_strsignal (signal_number));
+
+        gdm_slave_stopped (GDM_SLAVE (slave));
+}
+
 static gboolean
 gdm_simple_slave_run (GdmSimpleSlave *slave)
 {
@@ -752,10 +872,17 @@ gdm_simple_slave_run (GdmSimpleSlave *slave)
                 gboolean res;
 
                 slave->priv->server = gdm_server_new (display_name, auth_file);
-
+                g_signal_connect (slave->priv->server,
+                                  "exited",
+                                  G_CALLBACK (on_server_exited),
+                                  slave);
+                g_signal_connect (slave->priv->server,
+                                  "died",
+                                  G_CALLBACK (on_server_died),
+                                  slave);
                 g_signal_connect (slave->priv->server,
                                   "ready",
-                                  G_CALLBACK (server_ready_cb),
+                                  G_CALLBACK (on_server_ready),
                                   slave);
 
                 res = gdm_server_start (slave->priv->server);
@@ -785,9 +912,7 @@ gdm_simple_slave_run (GdmSimpleSlave *slave)
 static gboolean
 gdm_simple_slave_start (GdmSlave *slave)
 {
-        gboolean res;
-
-        res = GDM_SLAVE_CLASS (gdm_simple_slave_parent_class)->start (slave);
+        GDM_SLAVE_CLASS (gdm_simple_slave_parent_class)->start (slave);
 
         gdm_simple_slave_run (GDM_SIMPLE_SLAVE (slave));
 
@@ -797,14 +922,12 @@ gdm_simple_slave_start (GdmSlave *slave)
 static gboolean
 gdm_simple_slave_stop (GdmSlave *slave)
 {
-        gboolean res;
-
         g_debug ("GdmSimpleSlave: Stopping simple_slave");
 
-        res = GDM_SLAVE_CLASS (gdm_simple_slave_parent_class)->stop (slave);
+        GDM_SLAVE_CLASS (gdm_simple_slave_parent_class)->stop (slave);
 
         if (GDM_SIMPLE_SLAVE (slave)->priv->greeter != NULL) {
-                gdm_greeter_session_stop (GDM_SIMPLE_SLAVE (slave)->priv->greeter);
+                gdm_welcome_session_stop (GDM_WELCOME_SESSION (GDM_SIMPLE_SLAVE (slave)->priv->greeter));
                 g_object_unref (GDM_SIMPLE_SLAVE (slave)->priv->greeter);
                 GDM_SIMPLE_SLAVE (slave)->priv->greeter = NULL;
         }
@@ -830,10 +953,6 @@ gdm_simple_slave_set_property (GObject      *object,
                                const GValue *value,
                                GParamSpec   *pspec)
 {
-        GdmSimpleSlave *self;
-
-        self = GDM_SIMPLE_SLAVE (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -847,10 +966,6 @@ gdm_simple_slave_get_property (GObject    *object,
                                GValue      *value,
                                GParamSpec *pspec)
 {
-        GdmSimpleSlave *self;
-
-        self = GDM_SIMPLE_SLAVE (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -864,9 +979,6 @@ gdm_simple_slave_constructor (GType                  type,
                               GObjectConstructParam *construct_properties)
 {
         GdmSimpleSlave      *simple_slave;
-        GdmSimpleSlaveClass *klass;
-
-        klass = GDM_SIMPLE_SLAVE_CLASS (g_type_class_peek (GDM_TYPE_SIMPLE_SLAVE));
 
         simple_slave = GDM_SIMPLE_SLAVE (G_OBJECT_CLASS (gdm_simple_slave_parent_class)->constructor (type,
                                                                                  n_construct_properties,
