@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2007 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2008 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,6 +50,7 @@
 #include <gtk/gtk.h>
 
 #include <glade/glade-xml.h>
+#include <gconf/gconf-client.h>
 
 #define DBUS_API_SUBJECT_TO_CHANGE
 #include <dbus/dbus-glib.h>
@@ -58,11 +60,13 @@
 #include <polkit-gnome/polkit-gnome.h>
 #endif
 
+#include "gdm-settings-client.h"
+#include "gdm-settings-keys.h"
+
 #include "gdm-greeter-login-window.h"
 #include "gdm-user-chooser-widget.h"
-#include "gdm-session-chooser-widget.h"
 
-#if HAVE_PAM
+#ifdef HAVE_PAM
 #include <security/pam_appl.h>
 #define PW_ENTRY_SIZE PAM_MAX_RESP_SIZE
 #else
@@ -78,7 +82,15 @@
 #define CK_SEAT_INTERFACE    "org.freedesktop.ConsoleKit.Seat"
 #define CK_SESSION_INTERFACE "org.freedesktop.ConsoleKit.Session"
 
-#define GLADE_XML_FILE "gdm-greeter-login-window.glade"
+#define GPM_DBUS_NAME      "org.freedesktop.PowerManagement"
+#define GPM_DBUS_PATH      "/org/freedesktop/PowerManagement"
+#define GPM_DBUS_INTERFACE "org.freedesktop.PowerManagement"
+
+#define GLADE_XML_FILE       "gdm-greeter-login-window.glade"
+
+#define KEY_LOGO                    "/apps/gdm/simple-greeter/logo-icon-name"
+#define KEY_DISABLE_RESTART_BUTTONS "/apps/gdm/simple-greeter/disable-restart-buttons"
+#define TIMED_LOGIN_TIMEOUT_SEC  60
 
 #define GDM_GREETER_LOGIN_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_GREETER_LOGIN_WINDOW, GdmGreeterLoginWindowPrivate))
 
@@ -87,17 +99,26 @@ enum {
         MODE_AUTHENTICATION
 };
 
+enum {
+        LOGIN_BUTTON_HIDDEN = 0,
+        LOGIN_BUTTON_ANSWER_QUERY,
+        LOGIN_BUTTON_TIMED_LOGIN
+};
+
 struct GdmGreeterLoginWindowPrivate
 {
         GladeXML        *xml;
-        GtkWidget       *session_chooser;
         GtkWidget       *user_chooser;
         gboolean         display_is_local;
         char            *timeformat;
-        guint            update_clock_id;
-        gboolean         clock_show_seconds;
+
+        gboolean         timed_login_enabled;
+        guint            timed_login_delay;
+        guint            timed_login_timeout_id;
 
         guint            animation_timeout_id;
+
+        guint            login_button_handler_id;
 };
 
 enum {
@@ -106,13 +127,11 @@ enum {
 };
 
 enum {
+        BEGIN_TIMED_LOGIN,
         BEGIN_VERIFICATION,
         BEGIN_VERIFICATION_FOR_USER,
         QUERY_ANSWER,
-        SESSION_SELECTED,
-        LANGUAGE_SELECTED,
         USER_SELECTED,
-        HOSTNAME_SELECTED,
         DISCONNECTED,
         CANCELLED,
         LAST_SIGNAL
@@ -124,8 +143,6 @@ static void     gdm_greeter_login_window_class_init   (GdmGreeterLoginWindowClas
 static void     gdm_greeter_login_window_init         (GdmGreeterLoginWindow      *greeter_login_window);
 static void     gdm_greeter_login_window_finalize     (GObject                    *object);
 
-static gboolean update_clock_timeout_cb               (GdmGreeterLoginWindow *login_window);
-
 G_DEFINE_TYPE (GdmGreeterLoginWindow, gdm_greeter_login_window, GTK_TYPE_WINDOW)
 
 static void
@@ -175,7 +192,6 @@ set_focus (GdmGreeterLoginWindow *login_window)
         }
 }
 
-
 static void
 set_message (GdmGreeterLoginWindow *login_window,
              const char            *text)
@@ -186,6 +202,43 @@ set_message (GdmGreeterLoginWindow *login_window,
         gtk_label_set_text (GTK_LABEL (label), text);
 }
 
+static void
+remove_timed_login_timeout (GdmGreeterLoginWindow *login_window)
+{
+        if (login_window->priv->timed_login_timeout_id > 0) {
+                g_debug ("GdmGreeterLoginWindow: removing timed login timer");
+                g_source_remove (login_window->priv->timed_login_timeout_id);
+                login_window->priv->timed_login_timeout_id = 0;
+        }
+}
+
+static gboolean
+timed_login_timer (GdmGreeterLoginWindow *login_window)
+{
+        set_sensitive (login_window, FALSE);
+        set_message (login_window, _("Automatically logging in..."));
+
+        g_debug ("GdmGreeterLoginWindow: emitting begin-timed_login");
+        g_signal_emit (login_window, signals[BEGIN_TIMED_LOGIN], 0);
+        login_window->priv->timed_login_timeout_id = 0;
+
+        return FALSE;
+}
+
+static void
+restart_timed_login_timeout (GdmGreeterLoginWindow *login_window)
+{
+        remove_timed_login_timeout (login_window);
+
+        if (login_window->priv->timed_login_enabled) {
+                g_debug ("GdmGreeterLoginWindow: adding timed login timer");
+
+                login_window->priv->timed_login_timeout_id = g_timeout_add_seconds (login_window->priv->timed_login_delay,
+                                                                                    (GSourceFunc)timed_login_timer,
+                                                                                    login_window);
+        }
+}
+
 static void
 show_widget (GdmGreeterLoginWindow *login_window,
              const char            *name,
@@ -206,9 +259,18 @@ show_widget (GdmGreeterLoginWindow *login_window,
 static gboolean
 get_show_restart_buttons (GdmGreeterLoginWindow *login_window)
 {
-        gboolean show;
+        gboolean     show;
+        GConfClient *client;
+        GError      *error;
 
-        show = TRUE;
+        client = gconf_client_get_default ();
+        error = NULL;
+        show = ! gconf_client_get_bool (client, KEY_DISABLE_RESTART_BUTTONS, &error);
+        if (error != NULL) {
+                g_debug ("GdmGreeterLoginWindow: unable to get disable-restart-buttons configuration: %s", error->message);
+                g_error_free (error);
+        }
+        g_object_unref (client);
 
 #ifdef ENABLE_RBAC_SHUTDOWN
         {
@@ -217,10 +279,10 @@ get_show_restart_buttons (GdmGreeterLoginWindow *login_window)
                 username = g_get_user_name ();
                 if (username == NULL || !chkauthattr (RBAC_SHUTDOWN_KEY, username)) {
                         show = FALSE;
-                        g_debug ("Not showing stop/restart buttons for user %s due to RBAC key %s",
+                        g_debug ("GdmGreeterLoginWindow: Not showing stop/restart buttons for user %s due to RBAC key %s",
                                  username, RBAC_SHUTDOWN_KEY);
                 } else {
-                        g_debug ("Showing stop/restart buttons for user %s due to RBAC key %s",
+                        g_debug ("GdmGreeterLoginWindow: Showing stop/restart buttons for user %s due to RBAC key %s",
                                  username, RBAC_SHUTDOWN_KEY);
                 }
         }
@@ -228,6 +290,65 @@ get_show_restart_buttons (GdmGreeterLoginWindow *login_window)
         return show;
 }
 
+static void
+on_login_button_clicked_answer_query (GtkButton             *button,
+                                      GdmGreeterLoginWindow *login_window)
+{
+        GtkWidget  *entry;
+        const char *text;
+
+        set_busy (login_window);
+        set_sensitive (login_window, FALSE);
+
+        entry = glade_xml_get_widget (login_window->priv->xml, "auth-prompt-entry");
+        text = gtk_entry_get_text (GTK_ENTRY (entry));
+
+        g_signal_emit (login_window, signals[QUERY_ANSWER], 0, text);
+}
+
+static void
+on_login_button_clicked_timed_login (GtkButton             *button,
+                                   GdmGreeterLoginWindow *login_window)
+{
+        set_busy (login_window);
+        set_sensitive (login_window, FALSE);
+
+        g_signal_emit (login_window, signals[BEGIN_TIMED_LOGIN], 0);
+}
+
+static void
+set_log_in_button_mode (GdmGreeterLoginWindow *login_window,
+                        int                    mode)
+{
+        GtkWidget *button;
+
+        button = glade_xml_get_widget (login_window->priv->xml, "log-in-button");
+        gtk_widget_grab_default (button);
+
+        /* disconnect any signals */
+        if (login_window->priv->login_button_handler_id > 0) {
+                g_signal_handler_disconnect (button, login_window->priv->login_button_handler_id);
+                login_window->priv->login_button_handler_id = 0;
+       }
+
+        switch (mode) {
+        case LOGIN_BUTTON_HIDDEN:
+                gtk_widget_hide (button);
+                break;
+        case LOGIN_BUTTON_ANSWER_QUERY:
+                login_window->priv->login_button_handler_id = g_signal_connect (button, "clicked", G_CALLBACK (on_login_button_clicked_answer_query), login_window);
+                gtk_widget_show (button);
+                break;
+        case LOGIN_BUTTON_TIMED_LOGIN:
+                login_window->priv->login_button_handler_id = g_signal_connect (button, "clicked", G_CALLBACK (on_login_button_clicked_timed_login), login_window);
+                gtk_widget_show (button);
+                break;
+        default:
+                g_assert_not_reached ();
+                break;
+        }
+}
+
 static void
 switch_mode (GdmGreeterLoginWindow *login_window,
              int                    number)
@@ -235,36 +356,37 @@ switch_mode (GdmGreeterLoginWindow *login_window,
         const char *default_name;
         GtkWidget  *user_chooser;
         GtkWidget  *box;
-        gboolean    show_restart_shutdown = TRUE;
+        gboolean    show_restart_buttons;
 
         /* FIXME: do animation */
         default_name = NULL;
 
-        show_restart_shutdown = get_show_restart_buttons (login_window);
+        show_restart_buttons = get_show_restart_buttons (login_window);
 
         switch (number) {
         case MODE_SELECTION:
-                show_widget (login_window, "log-in-button", FALSE);
+                restart_timed_login_timeout (login_window);
+
+                set_log_in_button_mode (login_window, LOGIN_BUTTON_HIDDEN);
+
                 show_widget (login_window, "cancel-button", FALSE);
                 show_widget (login_window, "shutdown-button",
-                             login_window->priv->display_is_local && show_restart_shutdown);
+                             login_window->priv->display_is_local && show_restart_buttons);
                 show_widget (login_window, "restart-button",
-                             login_window->priv->display_is_local && show_restart_shutdown);
-                show_widget (login_window, "suspend-button", login_window->priv->display_is_local);
-                show_widget (login_window, "disconnect-button", ! login_window->priv->display_is_local);
+                             login_window->priv->display_is_local && show_restart_buttons);
+                show_widget (login_window, "suspend-button",
+                             login_window->priv->display_is_local && show_restart_buttons);
+                show_widget (login_window, "disconnect-button",
+                             ! login_window->priv->display_is_local);
                 show_widget (login_window, "auth-input-box", FALSE);
-                show_widget (login_window, "session-chooser", FALSE);
                 default_name = NULL;
                 break;
         case MODE_AUTHENTICATION:
-                show_widget (login_window, "log-in-button", TRUE);
                 show_widget (login_window, "cancel-button", TRUE);
                 show_widget (login_window, "shutdown-button", FALSE);
                 show_widget (login_window, "restart-button", FALSE);
                 show_widget (login_window, "suspend-button", FALSE);
                 show_widget (login_window, "disconnect-button", FALSE);
-                show_widget (login_window, "auth-input-box", TRUE);
-                show_widget (login_window, "session-chooser", TRUE);
                 default_name = "log-in-button";
                 break;
         default:
@@ -303,17 +425,57 @@ switch_mode (GdmGreeterLoginWindow *login_window,
         }
 }
 
+static void
+do_disconnect (GdmGreeterLoginWindow *login_window)
+{
+        gtk_main_quit ();
+}
+
+static void
+do_suspend (GdmGreeterLoginWindow *login_window)
+{
+        GError          *error;
+        DBusGConnection *connection;
+        DBusGProxy      *proxy;
+
+        g_debug ("GdmGreeterLoginWindow: Suspend button clicked");
+
+        error = NULL;
+        connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+        if (error != NULL) {
+                g_warning ("Couldn't suspend: %s", error->message);
+                g_error_free (error);
+                return;
+        }
+        proxy = dbus_g_proxy_new_for_name (connection,
+                                           GPM_DBUS_NAME,
+                                           GPM_DBUS_PATH,
+                                           GPM_DBUS_INTERFACE);
+        error = NULL;
+        dbus_g_proxy_call (proxy,
+                           "Suspend",
+                           &error,
+                           G_TYPE_INVALID,
+                           G_TYPE_INVALID);
+        if (error != NULL) {
+                g_warning ("Couldn't suspend: %s", error->message);
+                g_error_free (error);
+                return;
+        }
+
+        g_object_unref (proxy);
+}
+
 static void
 do_cancel (GdmGreeterLoginWindow *login_window)
 {
 
         gdm_user_chooser_widget_set_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), NULL);
-        gdm_session_chooser_widget_set_current_session_name (GDM_SESSION_CHOOSER_WIDGET (login_window->priv->session_chooser),
-                                                             GDM_SESSION_CHOOSER_SESSION_PREVIOUS);
 
         switch_mode (login_window, MODE_SELECTION);
         set_busy (login_window);
         set_sensitive (login_window, FALSE);
+        set_message (login_window, "");
 
         g_signal_emit (login_window, signals[CANCELLED], 0);
 
@@ -331,6 +493,7 @@ reset_dialog (GdmGreeterLoginWindow *login_window)
         entry = glade_xml_get_widget (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->xml, "auth-prompt-entry");
         gtk_entry_set_text (GTK_ENTRY (entry), "");
         gtk_entry_set_visibility (GTK_ENTRY (entry), TRUE);
+        set_message (login_window, "");
 
         label = glade_xml_get_widget (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->xml, "auth-prompt-label");
         gtk_label_set_text (GTK_LABEL (label), "");
@@ -338,14 +501,14 @@ reset_dialog (GdmGreeterLoginWindow *login_window)
         set_message (login_window, "");
 
         gdm_user_chooser_widget_set_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), NULL);
-        gdm_session_chooser_widget_set_current_session_name (GDM_SESSION_CHOOSER_WIDGET (login_window->priv->session_chooser),
-                                                             GDM_SESSION_CHOOSER_SESSION_PREVIOUS);
 
         switch_mode (login_window, MODE_SELECTION);
 
         set_sensitive (login_window, TRUE);
         set_ready (login_window);
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
+
+        restart_timed_login_timeout (login_window);
 }
 
 gboolean
@@ -410,10 +573,12 @@ gdm_greeter_login_window_info_query (GdmGreeterLoginWindow *login_window,
         entry = glade_xml_get_widget (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->xml, "auth-prompt-entry");
         gtk_entry_set_text (GTK_ENTRY (entry), "");
         gtk_entry_set_visibility (GTK_ENTRY (entry), TRUE);
+        set_log_in_button_mode (login_window, LOGIN_BUTTON_ANSWER_QUERY);
 
         label = glade_xml_get_widget (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->xml, "auth-prompt-label");
         gtk_label_set_text (GTK_LABEL (label), text);
 
+        show_widget (login_window, "auth-input-box", TRUE);
         set_sensitive (GDM_GREETER_LOGIN_WINDOW (login_window), TRUE);
         set_ready (GDM_GREETER_LOGIN_WINDOW (login_window));
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
@@ -433,10 +598,12 @@ gdm_greeter_login_window_secret_info_query (GdmGreeterLoginWindow *login_window,
         entry = glade_xml_get_widget (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->xml, "auth-prompt-entry");
         gtk_entry_set_text (GTK_ENTRY (entry), "");
         gtk_entry_set_visibility (GTK_ENTRY (entry), FALSE);
+        set_log_in_button_mode (login_window, LOGIN_BUTTON_ANSWER_QUERY);
 
         label = glade_xml_get_widget (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->xml, "auth-prompt-label");
         gtk_label_set_text (GTK_LABEL (label), text);
 
+        show_widget (login_window, "auth-input-box", TRUE);
         set_sensitive (GDM_GREETER_LOGIN_WINDOW (login_window), TRUE);
         set_ready (GDM_GREETER_LOGIN_WINDOW (login_window));
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
@@ -492,21 +659,13 @@ gdm_greeter_login_window_get_property (GObject    *object,
 }
 
 static void
-log_in_button_clicked (GtkButton             *button,
-                       GdmGreeterLoginWindow *login_window)
+suspend_button_clicked (GtkButton             *button,
+                        GdmGreeterLoginWindow *login_window)
 {
-        GtkWidget  *entry;
-        const char *text;
-
-        set_busy (login_window);
-        set_sensitive (login_window, FALSE);
-
-        entry = glade_xml_get_widget (login_window->priv->xml, "auth-prompt-entry");
-        text = gtk_entry_get_text (GTK_ENTRY (entry));
-
-        g_signal_emit (login_window, signals[QUERY_ANSWER], 0, text);
+        do_suspend (login_window);
 }
 
+
 static void
 cancel_button_clicked (GtkButton             *button,
                        GdmGreeterLoginWindow *login_window)
@@ -514,6 +673,13 @@ cancel_button_clicked (GtkButton             *button,
         do_cancel (login_window);
 }
 
+static void
+disconnect_button_clicked (GtkButton             *button,
+                           GdmGreeterLoginWindow *login_window)
+{
+        do_disconnect (login_window);
+}
+
 static gboolean
 try_system_stop (DBusGConnection *connection,
                  GError         **error)
@@ -645,7 +811,7 @@ get_action_from_error (GError *error)
         if (g_str_has_prefix (error->message, "Not privileged for action: ")) {
                 paction = error->message + strlen ("Not privileged for action: ");
         }
-        g_debug ("Requesting priv for '%s'", paction);
+        g_debug ("GdmGreeterLoginWindow: Requesting priv for '%s'", paction);
 
         polkit_action_set_action_id (action, paction);
 
@@ -784,11 +950,24 @@ on_user_chosen (GdmUserChooserWidget  *user_chooser,
                 return;
         }
 
+        g_signal_emit (G_OBJECT (login_window), signals[USER_SELECTED],
+                       0, user_name);
+
         if (strcmp (user_name, GDM_USER_CHOOSER_USER_OTHER) == 0) {
+                /* stop the timed login until the verification times
+                   out or is cancelled */
+                remove_timed_login_timeout (login_window);
                 g_signal_emit (login_window, signals[BEGIN_VERIFICATION], 0);
         } else if (strcmp (user_name, GDM_USER_CHOOSER_USER_GUEST) == 0) {
                 /* FIXME: handle guest account stuff */
+        } else if (strcmp (user_name, GDM_USER_CHOOSER_USER_AUTO) == 0) {
+                /* just wait for the user to select language and stuff */
+                set_log_in_button_mode (login_window, LOGIN_BUTTON_TIMED_LOGIN);
+                set_message (login_window, _("Select language and click Log In"));
         } else {
+                /* stop the timed login until the verification times
+                   out or is cancelled */
+                remove_timed_login_timeout (login_window);
                 g_signal_emit (login_window, signals[BEGIN_VERIFICATION_FOR_USER], 0, user_name);
         }
 
@@ -804,99 +983,6 @@ on_user_unchosen (GdmUserChooserWidget  *user_chooser,
         do_cancel (login_window);
 }
 
-static void
-on_session_activated (GdmSessionChooserWidget *session_chooser,
-                      GdmGreeterLoginWindow   *login_window)
-{
-        char *session;
-
-        session = gdm_session_chooser_widget_get_current_session_name (GDM_SESSION_CHOOSER_WIDGET (login_window->priv->session_chooser));
-        if (session == NULL) {
-                return;
-        }
-
-        g_signal_emit (login_window, signals[SESSION_SELECTED], 0, session);
-
-        g_free (session);
-}
-
-static void
-update_clock (GtkLabel   *label,
-              const char *format)
-{
-        time_t     t;
-        struct tm *tm;
-        char       buf[256];
-        char      *utf8;
-
-        time (&t);
-        tm = localtime (&t);
-        if (tm == NULL) {
-                g_warning ("Unable to get broken down local time");
-                return;
-        }
-        if (strftime (buf, sizeof (buf), format, tm) == 0) {
-                g_warning ("Couldn't format time: %s", format);
-                strcpy (buf, "???");
-        }
-        utf8 = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
-        gtk_label_set_text (label, utf8);
-        g_free (utf8);
-}
-
-static void
-set_clock_timeout (GdmGreeterLoginWindow *login_window,
-                   time_t                 now)
-{
-        struct timeval tv;
-        int            timeouttime;
-
-        if (login_window->priv->update_clock_id > 0) {
-                g_source_remove (login_window->priv->update_clock_id);
-                login_window->priv->update_clock_id = 0;
-        }
-
-        gettimeofday (&tv, NULL);
-        timeouttime = (G_USEC_PER_SEC - tv.tv_usec) / 1000 + 1;
-
-        /* timeout of one minute if we don't care about the seconds */
-        if (! login_window->priv->clock_show_seconds) {
-                timeouttime += 1000 * (59 - now % 60);
-        }
-
-        login_window->priv->update_clock_id = g_timeout_add (timeouttime,
-                                                             (GSourceFunc)update_clock_timeout_cb,
-                                                             login_window);
-
-}
-
-static gboolean
-update_clock_timeout_cb (GdmGreeterLoginWindow *login_window)
-{
-        GtkWidget *label;
-        time_t     new_time;
-
-        time (&new_time);
-
-        label = glade_xml_get_widget (login_window->priv->xml, "computer-info-time-label");
-        if (label != NULL) {
-                update_clock (GTK_LABEL (label), login_window->priv->timeformat);
-        }
-
-        set_clock_timeout (login_window, new_time);
-
-        return FALSE;
-}
-
-static void
-remove_clock_timeout (GdmGreeterLoginWindow *login_window)
-{
-        if (login_window->priv->update_clock_id > 0) {
-                g_source_remove (login_window->priv->update_clock_id);
-                login_window->priv->update_clock_id = 0;
-        }
-}
-
 static gboolean
 on_computer_info_label_button_press (GtkWidget             *widget,
                                      GdkEventButton        *event,
@@ -905,7 +991,6 @@ on_computer_info_label_button_press (GtkWidget             *widget,
         GtkWidget *notebook;
         int        current_page;
         int        n_pages;
-        GtkWidget *label;
 
         /* switch page */
         notebook = glade_xml_get_widget (login_window->priv->xml, "computer-info-notebook");
@@ -918,16 +1003,6 @@ on_computer_info_label_button_press (GtkWidget             *widget,
                 gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 0);
         }
 
-        /* if the clock is visible then start it */
-        label = glade_xml_get_widget (login_window->priv->xml, "computer-info-time-label");
-        if (gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)) == gtk_notebook_page_num (GTK_NOTEBOOK (notebook), label)) {
-                time_t now;
-                time (&now);
-                set_clock_timeout (login_window, now);
-        } else {
-                remove_clock_timeout (login_window);
-        }
-
         return FALSE;
 }
 
@@ -1008,35 +1083,6 @@ get_system_version (void)
         return version;
 }
 
-static char *
-get_time_format (GdmGreeterLoginWindow *login_window)
-{
-        const char *time_format;
-        const char *date_format;
-        char       *clock_format;
-        char       *result;
-
-        time_format = login_window->priv->clock_show_seconds ? _("%l:%M:%S %p") : _("%l:%M %p");
-        /* translators: replace %e with %d if, when the day of the
-         *              month as a decimal number is a single digit, it
-         *              should begin with a 0 in your locale (e.g. "May
-         *              01" instead of "May  1").
-         */
-        date_format = _("%a %b %e");
-        /* translators: reverse the order of these arguments
-         *              if the time should come before the
-         *              date on a clock in your locale.
-         */
-        clock_format = g_strdup_printf (_("%1$s, %2$s"),
-                                        date_format,
-                                        time_format);
-
-        result = g_locale_from_utf8 (clock_format, -1, NULL, NULL, NULL);
-        g_free (clock_format);
-
-        return result;
-}
-
 static void
 create_computer_info (GdmGreeterLoginWindow *login_window)
 {
@@ -1054,13 +1100,6 @@ create_computer_info (GdmGreeterLoginWindow *login_window)
                 gtk_label_set_text (GTK_LABEL (label), version);
                 g_free (version);
         }
-
-
-        label = glade_xml_get_widget (login_window->priv->xml, "computer-info-time-label");
-        if (label != NULL) {
-                login_window->priv->timeformat = get_time_format (login_window);
-                update_clock (GTK_LABEL (label), login_window->priv->timeformat);
-        }
 }
 
 #define INVISIBLE_CHAR_DEFAULT       '*'
@@ -1089,8 +1128,6 @@ custom_widget_constructor (GladeXML              *xml,
 
         if (strcmp (name, "user-chooser") == 0) {
                widget = gdm_user_chooser_widget_new ();
-        } else if (strcmp (name, "session-chooser") == 0) {
-               widget = gdm_session_chooser_widget_new ();
         }
 
         return widget;
@@ -1102,6 +1139,7 @@ load_theme (GdmGreeterLoginWindow *login_window)
         GtkWidget *entry;
         GtkWidget *button;
         GtkWidget *box;
+        GtkWidget *image;
 
         glade_set_custom_handler ((GladeXMLCustomWidgetHandler) custom_widget_constructor,
                                   login_window);
@@ -1111,6 +1149,29 @@ load_theme (GdmGreeterLoginWindow *login_window)
 
         g_assert (login_window->priv->xml != NULL);
 
+        image = glade_xml_get_widget (login_window->priv->xml, "logo-image");
+        if (image != NULL) {
+                GConfClient *client;
+                char        *icon_name;
+                GError      *error;
+
+                client = gconf_client_get_default ();
+                error = NULL;
+                icon_name = gconf_client_get_string (client, KEY_LOGO, &error);
+                if (error != NULL) {
+                        g_debug ("GdmGreeterLoginWindow: unable to get logo icon name: %s", error->message);
+                        g_error_free (error);
+                }
+                g_object_unref (client);
+                g_debug ("GdmGreeterLoginWindow: Got greeter logo '%s'", icon_name);
+                if (icon_name != NULL) {
+                        gtk_image_set_from_icon_name (GTK_IMAGE (image),
+                                                      icon_name,
+                                                      GTK_ICON_SIZE_DIALOG);
+                        g_free (icon_name);
+                }
+        }
+
         box = glade_xml_get_widget (login_window->priv->xml, "window-box");
         gtk_container_add (GTK_CONTAINER (login_window), box);
 
@@ -1123,9 +1184,9 @@ load_theme (GdmGreeterLoginWindow *login_window)
 
         gdm_user_chooser_widget_set_show_only_chosen (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), TRUE);
 
-        /* FIXME: set from gconf */
-        gdm_user_chooser_widget_set_show_other_user (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), TRUE);
-        gdm_user_chooser_widget_set_show_guest_user (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), TRUE);
+        if (login_window->priv->timed_login_enabled) {
+                gdm_user_chooser_widget_set_show_auto_user (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), TRUE);
+        }
 
         g_signal_connect (login_window->priv->user_chooser,
                           "activated",
@@ -1138,31 +1199,15 @@ load_theme (GdmGreeterLoginWindow *login_window)
 
         gtk_widget_show (login_window->priv->user_chooser);
 
-        login_window->priv->session_chooser =
-                glade_xml_get_widget (login_window->priv->xml, "session-chooser");
-
-        if (login_window->priv->session_chooser == NULL) {
-                g_critical ("Session chooser not found in greeter theme");
-        }
-
-        gdm_session_chooser_widget_set_show_only_chosen (GDM_SESSION_CHOOSER_WIDGET (login_window->priv->session_chooser), TRUE);
-
-        g_signal_connect (login_window->priv->session_chooser,
-                          "activated",
-                          G_CALLBACK (on_session_activated),
-                          login_window);
-
-        gdm_session_chooser_widget_set_current_session_name (GDM_SESSION_CHOOSER_WIDGET (login_window->priv->session_chooser),
-                                                             GDM_SESSION_CHOOSER_SESSION_PREVIOUS);
-
-
-        button = glade_xml_get_widget (login_window->priv->xml, "log-in-button");
-        gtk_widget_grab_default (button);
-        g_signal_connect (button, "clicked", G_CALLBACK (log_in_button_clicked), login_window);
+        button = glade_xml_get_widget (login_window->priv->xml, "suspend-button");
+        g_signal_connect (button, "clicked", G_CALLBACK (suspend_button_clicked), login_window);
 
         button = glade_xml_get_widget (login_window->priv->xml, "cancel-button");
         g_signal_connect (button, "clicked", G_CALLBACK (cancel_button_clicked), login_window);
 
+        button = glade_xml_get_widget (login_window->priv->xml, "disconnect-button");
+        g_signal_connect (button, "clicked", G_CALLBACK (disconnect_button_clicked), login_window);
+
         button = glade_xml_get_widget (login_window->priv->xml, "restart-button");
         g_signal_connect (button, "clicked", G_CALLBACK (restart_button_clicked), login_window);
         button = glade_xml_get_widget (login_window->priv->xml, "shutdown-button");
@@ -1292,21 +1337,62 @@ gdm_greeter_login_window_size_allocate (GtkWidget      *widget,
         GTK_WIDGET_CLASS (gdm_greeter_login_window_parent_class)->size_allocate (widget, allocation);
 }
 
+static void
+read_configuration (GdmGreeterLoginWindow *login_window)
+{
+        gboolean res;
+        int      delay;
+        char    *username;
+
+        g_debug ("GdmGreeterLoginWindow: reading system configuration");
+
+        res = gdm_settings_client_get_boolean (GDM_KEY_TIMED_LOGIN_ENABLE,
+                                               &login_window->priv->timed_login_enabled);
+        if (! res) {
+                g_warning ("Unable to read configuration for %s", GDM_KEY_TIMED_LOGIN_ENABLE);
+        }
+        g_debug ("GdmGreeterLoginWindow: TimedLoginEnable=%s",
+                 login_window->priv->timed_login_enabled ? "true" : "false");
+
+        /* treat failures here as disabling */
+        username = NULL;
+        res = gdm_settings_client_get_string (GDM_KEY_TIMED_LOGIN_USER, &username);
+        if (! res) {
+                g_warning ("Unable to read configuration for %s", GDM_KEY_TIMED_LOGIN_USER);
+                login_window->priv->timed_login_enabled = FALSE;
+        } else {
+                if (username == NULL) {
+                        login_window->priv->timed_login_enabled = FALSE;
+                }
+        }
+        g_debug ("GdmGreeterLoginWindow: TimedLogin=%s", username);
+        g_free (username);
+
+        delay = -1;
+        res = gdm_settings_client_get_int (GDM_KEY_TIMED_LOGIN_DELAY, &delay);
+        if (! res) {
+                g_warning ("Unable to read configuration for %s", GDM_KEY_TIMED_LOGIN_DELAY);
+        } else {
+                if (delay >= 0) {
+                        login_window->priv->timed_login_delay = delay;
+                }
+        }
+        g_debug ("GdmGreeterLoginWindow: TimedLoginDelay=%d", delay);
+}
+
 static GObject *
 gdm_greeter_login_window_constructor (GType                  type,
                                       guint                  n_construct_properties,
                                       GObjectConstructParam *construct_properties)
 {
         GdmGreeterLoginWindow      *login_window;
-        GdmGreeterLoginWindowClass *klass;
-
-        klass = GDM_GREETER_LOGIN_WINDOW_CLASS (g_type_class_peek (GDM_TYPE_GREETER_LOGIN_WINDOW));
 
         login_window = GDM_GREETER_LOGIN_WINDOW (G_OBJECT_CLASS (gdm_greeter_login_window_parent_class)->constructor (type,
                                                                                                                       n_construct_properties,
                                                                                                                       construct_properties));
 
 
+        read_configuration (login_window);
         load_theme (login_window);
 
         return G_OBJECT (login_window);
@@ -1327,6 +1413,16 @@ gdm_greeter_login_window_class_init (GdmGreeterLoginWindowClass *klass)
         widget_class->size_request = gdm_greeter_login_window_size_request;
         widget_class->size_allocate = gdm_greeter_login_window_size_allocate;
 
+        signals [BEGIN_TIMED_LOGIN] =
+                g_signal_new ("begin-timed_login",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GdmGreeterLoginWindowClass, begin_timed_login),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0);
         signals [BEGIN_VERIFICATION] =
                 g_signal_new ("begin-verification",
                               G_TYPE_FROM_CLASS (object_class),
@@ -1357,16 +1453,6 @@ gdm_greeter_login_window_class_init (GdmGreeterLoginWindowClass *klass)
                               g_cclosure_marshal_VOID__STRING,
                               G_TYPE_NONE,
                               1, G_TYPE_STRING);
-        signals [LANGUAGE_SELECTED] =
-                g_signal_new ("language-selected",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdmGreeterLoginWindowClass, language_selected),
-                              NULL,
-                              NULL,
-                              g_cclosure_marshal_VOID__STRING,
-                              G_TYPE_NONE,
-                              1, G_TYPE_STRING);
         signals [USER_SELECTED] =
                 g_signal_new ("user-selected",
                               G_TYPE_FROM_CLASS (object_class),
@@ -1377,26 +1463,6 @@ gdm_greeter_login_window_class_init (GdmGreeterLoginWindowClass *klass)
                               g_cclosure_marshal_VOID__STRING,
                               G_TYPE_NONE,
                               1, G_TYPE_STRING);
-        signals [SESSION_SELECTED] =
-                g_signal_new ("session-selected",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdmGreeterLoginWindowClass, session_selected),
-                              NULL,
-                              NULL,
-                              g_cclosure_marshal_VOID__STRING,
-                              G_TYPE_NONE,
-                              1, G_TYPE_STRING);
-        signals [HOSTNAME_SELECTED] =
-                g_signal_new ("hostname-selected",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdmGreeterLoginWindowClass, hostname_selected),
-                              NULL,
-                              NULL,
-                              g_cclosure_marshal_VOID__STRING,
-                              G_TYPE_NONE,
-                              1, G_TYPE_STRING);
         signals [CANCELLED] =
                 g_signal_new ("cancelled",
                               G_TYPE_FROM_CLASS (object_class),
@@ -1433,8 +1499,10 @@ gdm_greeter_login_window_init (GdmGreeterLoginWindow *login_window)
 {
         login_window->priv = GDM_GREETER_LOGIN_WINDOW_GET_PRIVATE (login_window);
 
-        login_window->priv->clock_show_seconds = TRUE;
+        login_window->priv->timed_login_enabled = FALSE;
+        login_window->priv->timed_login_delay = TIMED_LOGIN_TIMEOUT_SEC;
 
+        gtk_window_set_title (GTK_WINDOW (login_window), _("Login Window"));
         gtk_window_set_opacity (GTK_WINDOW (login_window), 0.85);
         gtk_window_set_position (GTK_WINDOW (login_window), GTK_WIN_POS_CENTER_ALWAYS);
         gtk_window_set_deletable (GTK_WINDOW (login_window), FALSE);
@@ -1457,8 +1525,6 @@ gdm_greeter_login_window_finalize (GObject *object)
 
         g_return_if_fail (login_window->priv != NULL);
 
-        remove_clock_timeout (login_window);
-
         G_OBJECT_CLASS (gdm_greeter_login_window_parent_class)->finalize (object);
 }
 
