@@ -59,6 +59,16 @@ gdm_language_chooser_dialog_get_current_language_name (GdmLanguageChooserDialog
         return language_name;
 }
 
+void
+gdm_language_chooser_dialog_set_current_language_name (GdmLanguageChooserDialog *dialog,
+                                                       const char               *language_name)
+{
+
+        g_return_if_fail (GDM_IS_LANGUAGE_CHOOSER_DIALOG (dialog));
+
+        gdm_language_chooser_widget_set_current_language_name (GDM_LANGUAGE_CHOOSER_WIDGET (dialog->priv->chooser_widget), language_name);
+}
+
 static void
 gdm_language_chooser_dialog_size_request (GtkWidget      *widget,
                                        GtkRequisition *requisition)
@@ -113,6 +123,10 @@ gdm_language_chooser_dialog_init (GdmLanguageChooserDialog *dialog)
                                 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                 GTK_STOCK_OK, GTK_RESPONSE_OK,
                                 NULL);
+        gtk_window_set_icon_name (GTK_WINDOW (dialog), "config-language");
+        gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+        gtk_container_set_border_width (GTK_CONTAINER (dialog), 12);
+        gtk_container_set_border_width (GTK_CONTAINER (dialog->priv->chooser_widget), 5);
 }
 
 static void
