@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2004-2005 James M. Cape <jcape@ignore-your.tv>.
- * Copyright (C) 2007 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2007-2008 William Jon McCann <mccann@jhu.edu>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,8 +26,8 @@
 #include <unistd.h>
 
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 #include <gtk/gtk.h>
-#include <libgnomevfs/gnome-vfs.h>
 
 #include "gdm-user-manager.h"
 #include "gdm-user-private.h"
@@ -51,6 +51,7 @@ enum {
         PROP_UID,
         PROP_HOME_DIR,
         PROP_SHELL,
+        PROP_LOGIN_FREQUENCY,
 };
 
 enum {
@@ -69,7 +70,8 @@ struct _GdmUser {
         char           *real_name;
         char           *home_dir;
         char           *shell;
-        GSList         *sessions;
+        GList          *sessions;
+        gulong          login_frequency;
 };
 
 typedef struct _GdmUserClass
@@ -84,7 +86,7 @@ static void gdm_user_finalize     (GObject      *object);
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
-G_DEFINE_TYPE (GdmUser, gdm_user, G_TYPE_OBJECT);
+G_DEFINE_TYPE (GdmUser, gdm_user, G_TYPE_OBJECT)
 
 static int
 session_compare (const char *a,
@@ -103,15 +105,15 @@ void
 _gdm_user_add_session (GdmUser    *user,
                        const char *ssid)
 {
-        GSList *li;
+        GList *li;
 
         g_return_if_fail (GDM_IS_USER (user));
         g_return_if_fail (ssid != NULL);
 
-        li = g_slist_find_custom (user->sessions, ssid, (GCompareFunc)session_compare);
+        li = g_list_find_custom (user->sessions, ssid, (GCompareFunc)session_compare);
         if (li == NULL) {
                 g_debug ("GdmUser: adding session %s", ssid);
-                user->sessions = g_slist_prepend (user->sessions, g_strdup (ssid));
+                user->sessions = g_list_prepend (user->sessions, g_strdup (ssid));
                 g_signal_emit (user, signals[SESSIONS_CHANGED], 0);
         } else {
                 g_debug ("GdmUser: session already present: %s", ssid);
@@ -122,16 +124,16 @@ void
 _gdm_user_remove_session (GdmUser    *user,
                           const char *ssid)
 {
-        GSList *li;
+        GList *li;
 
         g_return_if_fail (GDM_IS_USER (user));
         g_return_if_fail (ssid != NULL);
 
-        li = g_slist_find_custom (user->sessions, ssid, (GCompareFunc)session_compare);
+        li = g_list_find_custom (user->sessions, ssid, (GCompareFunc)session_compare);
         if (li != NULL) {
                 g_debug ("GdmUser: removing session %s", ssid);
                 g_free (li->data);
-                user->sessions = g_slist_delete_link (user->sessions, li);
+                user->sessions = g_list_delete_link (user->sessions, li);
                 g_signal_emit (user, signals[SESSIONS_CHANGED], 0);
         } else {
                 g_debug ("GdmUser: session not found: %s", ssid);
@@ -141,7 +143,21 @@ _gdm_user_remove_session (GdmUser    *user,
 guint
 gdm_user_get_num_sessions (GdmUser    *user)
 {
-        return g_slist_length (user->sessions);
+        return g_list_length (user->sessions);
+}
+
+GList *
+gdm_user_get_sessions (GdmUser *user)
+{
+        return user->sessions;
+}
+
+static void
+_gdm_user_set_login_frequency (GdmUser *user,
+                               gulong   login_frequency)
+{
+        user->login_frequency = login_frequency;
+        g_object_notify (G_OBJECT (user), "login-frequency");
 }
 
 static void
@@ -159,7 +175,9 @@ gdm_user_set_property (GObject      *object,
                 user->manager = g_value_get_object (value);
                 g_assert (user->manager);
                 break;
-
+        case PROP_LOGIN_FREQUENCY:
+                _gdm_user_set_login_frequency (user, g_value_get_ulong (value));
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
                 break;
@@ -195,6 +213,9 @@ gdm_user_get_property (GObject    *object,
         case PROP_SHELL:
                 g_value_set_string (value, user->shell);
                 break;
+        case PROP_LOGIN_FREQUENCY:
+                g_value_set_ulong (value, user->login_frequency);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
                 break;
@@ -257,6 +278,15 @@ gdm_user_class_init (GdmUserClass *class)
                                                               "The shell for this user.",
                                                               NULL,
                                                               G_PARAM_READABLE));
+        g_object_class_install_property (gobject_class,
+                                         PROP_LOGIN_FREQUENCY,
+                                         g_param_spec_ulong ("login-frequency",
+                                                             "login frequency",
+                                                             "login frequency",
+                                                             0,
+                                                             G_MAXULONG,
+                                                             0,
+                                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
         signals [ICON_CHANGED] =
                 g_signal_new ("icon-changed",
@@ -507,6 +537,14 @@ gdm_user_get_shell (GdmUser *user)
         return user->shell;
 }
 
+gulong
+gdm_user_get_login_frequency (GdmUser *user)
+{
+        g_return_val_if_fail (GDM_IS_USER (user), 0);
+
+        return user->login_frequency;
+}
+
 gint
 gdm_user_collate (GdmUser *user1,
                   GdmUser *user2)
@@ -597,30 +635,61 @@ static GdkPixbuf *
 render_icon_from_home (GdmUser *user,
                        int      icon_size)
 {
-        GdkPixbuf *  retval;
-        char        *path;
-        GnomeVFSURI *uri;
-        gboolean     is_local;
-        gboolean     res;
+        GdkPixbuf  *retval;
+        char       *path;
+        GFile      *file;
+        GFileInfo  *file_info;
+        gboolean    is_local;
+        gboolean    is_autofs;
+        gboolean    res;
+        const char *filesystem_type;
 
         /* special case: look at parent of home to detect autofs
            this is so we don't try to trigger an automount */
         path = g_path_get_dirname (user->home_dir);
-        uri = gnome_vfs_uri_new (path);
-        is_local = gnome_vfs_uri_is_local (uri);
-        gnome_vfs_uri_unref (uri);
+        file = g_file_new_for_path (path);
+        file_info = g_file_query_filesystem_info (file,
+                                                  G_FILE_ATTRIBUTE_FILESYSTEM_TYPE,
+                                                  NULL,
+                                                  NULL);
+        if (file_info == NULL) {
+                g_free (path);
+                g_object_unref (file);
+                return NULL;
+        }
+        filesystem_type = g_file_info_get_attribute_string (file_info,
+                                                            G_FILE_ATTRIBUTE_FILESYSTEM_TYPE);
+        is_autofs = (filesystem_type != NULL && strcmp (filesystem_type, "autofs") == 0);
+        g_object_unref (file);
+        g_object_unref (file_info);
         g_free (path);
 
         /* now check that home dir itself is local */
-        if (is_local) {
-                uri = gnome_vfs_uri_new (user->home_dir);
-                is_local = gnome_vfs_uri_is_local (uri);
-                gnome_vfs_uri_unref (uri);
+        is_local = TRUE;
+        if (! is_autofs) {
+                file = g_file_new_for_path (user->home_dir);
+                file_info = g_file_query_filesystem_info (file,
+                                                          G_FILE_ATTRIBUTE_FILESYSTEM_TYPE,
+                                                          NULL,
+                                                          NULL);
+                if (file_info == NULL) {
+                        g_object_unref (file);
+                        return NULL;
+                }
+                filesystem_type = g_file_info_get_attribute_string (file_info,
+                                                                    G_FILE_ATTRIBUTE_FILESYSTEM_TYPE);
+                is_local = ((strcmp (filesystem_type, "nfs") != 0) &&
+                            (strcmp (filesystem_type, "afs") != 0) &&
+                            (strcmp (filesystem_type, "autofs") != 0) &&
+                            (strcmp (filesystem_type, "unknown") != 0) &&
+                            (strcmp (filesystem_type, "ncpfs") != 0));
+                g_object_unref (file_info);
+                g_object_unref (file);
         }
 
         /* only look at local home directories so we don't try to
            read from remote (e.g. NFS) volumes */
-        if (!is_local) {
+        if (! is_local) {
                 return NULL;
         }
 
